/* Nond-domination based selection routines */

# include <stdio.h>
# include <stdlib.h>
# include <math.h>

# include "global.h"
# include "rand.h"


int num_violations(individual * ind)
{
  int num=0;
  int i;

  for(i=0;i<ncon;i++)
    if (ind->constr[i]<0)
      num++;

  return num;
}

/* Routine to perform non-dominated sorting */
void fill_nondominated_sort (population *mixed_pop, population *new_pop, Ranker* ranker, int gen_num)
{
    int flag;
    int i, j;
    int end;
    int front_size;
    int trainsize=0;
    int rank=1;
    list *pool;
    list *elite;
    list *temp1, *temp2;
    pool = (list *)malloc(sizeof(list));
    elite = (list *)malloc(sizeof(list));
    front_size = 0;
    pool->index = -1;
    pool->parent = NULL;
    pool->child = NULL;
    elite->index = -1;
    elite->parent = NULL;
    elite->child = NULL;
    temp1 = pool;

    /* normalize objectives to a common range */
    /*  if(ranker) */
    /*       ranker->normalizeObjectives(mixed_pop, 2*popsize, nobj); */

    for (i=0; i<2*popsize; i++)
    {
        insert (temp1,i);
        temp1 = temp1->child;
    }
    i=0;
    do
    {
        temp1 = pool->child;
        insert (elite, temp1->index);
        front_size = 1;
        temp2 = elite->child;
        temp1 = del (temp1);
        temp1 = temp1->child;
        do
        {
            temp2 = elite->child;
            if (temp1==NULL)
            {
                break;
            }
            do
            {
                end = 0;
                flag = check_dominance (&(mixed_pop->ind[temp1->index]), &(mixed_pop->ind[temp2->index]));
                if (flag == 1)
                {
                    insert (pool, temp2->index);
                    temp2 = del (temp2);
                    front_size--;
                    temp2 = temp2->child;
                }
                if (flag == 0)
                {
                    temp2 = temp2->child;
                }
                if (flag == -1)
                {
                    end = 1;
                }
            }
            while (end!=1 && temp2!=NULL);
            if (flag == 0 || flag == 1)
            {
                insert (elite, temp1->index);
                front_size++;
                temp1 = del (temp1);
            }
            temp1 = temp1->child;
        }
        while (temp1 != NULL);
        temp2 = elite->child;
        j=i;
      
	if(ranker && ranker->useRanker(gen_num)) 
          // use trained ranker
          {
	    ranker->setPreferences(mixed_pop, temp2, nobj, true);	    
            ranker->preference_ranking(mixed_pop, temp2, rank, new_pop, popsize, nobj, &i);           
            assign_crowding_distance_indices (new_pop, j, i-1);
            rank+=1;
          }
	else if ( (i+front_size) <= popsize)
        {
	  do
            {
	      copy_ind (&mixed_pop->ind[temp2->index], &new_pop->ind[i]);
	      new_pop->ind[i].rank = rank;
	      new_pop->ind[i].preference=0.;
	      temp2 = temp2->child;
	      i+=1;
            }
	  while (temp2 != NULL);
	  assign_crowding_distance_indices (new_pop, j, i-1);
	  rank+=1;
        }
        else
	  {
            crowding_fill (mixed_pop, new_pop, i, front_size, elite);
            for (j=i; j<popsize; j++)
	      {
		new_pop->ind[j].preference=0.;
                new_pop->ind[j].rank = rank;
	      }
            i = popsize;
	  }
	
        temp2 = elite->child;
        do
        {
            temp2 = del (temp2);
            temp2 = temp2->child;
        }
        while (elite->child !=NULL);
    }
    while (i < popsize);
    while (pool!=NULL)
    {
        temp1 = pool;
        pool = pool->child;
        free (temp1);
    }
    while (elite!=NULL)
    {
        temp1 = elite;
        elite = elite->child;
        free (temp1);
    }

    if(ranker && (ranker->trainRanker(gen_num))){
      // train ranker according to user preference 
      ranker->setUserPreferences(new_pop, nobj, popsize, &trainsize);
      ranker->update(new_pop, 0, trainsize, nobj);
       // add ranker preference info 
      ranker->setPreferences(new_pop, trainsize, popsize, nobj, true);
    }
    
    if(ranker){
      printf("\n");
      printf("best rank\tbest preference\ttrue preference");
      for(j = 0; j < nobj; j++)
	printf("\tobj[%d]",j);
      printf("\tviol\n");
      i=0;
      printf("%d\t%f\t%f", new_pop->ind[i].rank, new_pop->ind[i].preference, 
	     ranker->userPreference(new_pop->ind[i].obj, nobj));
      for(j = 0; j < nobj; j++)
	printf("\t%f",new_pop->ind[i].obj[j]);	
      printf("\t%d\n",num_violations(&(new_pop->ind[i])));
      ranker->decrementQueriesNumber();
      
      // DEBUG
      if((gen_num % 100) == 0){
	printf("\n");
	printf("rank\tpreference\ttrue preference");
	for(j = 0; j < nobj; j++)
	  printf("\tobj[%d]",j);
	printf("\tviol\n");
	for (i=0; i<popsize; i++)
	  {
	    printf("%d\t%f\t%f", new_pop->ind[i].rank, new_pop->ind[i].preference, 
		   ranker->userPreference(new_pop->ind[i].obj, nobj));
	    for(j = 0; j < nobj; j++)
	      printf("\t%f",new_pop->ind[i].obj[j]);	
	    printf("\t%d\n",num_violations(&(new_pop->ind[i])));
	    ranker->decrementQueriesNumber();
	  }
      }
    }
    return;
}

/* Routine to fill a population with individuals in the decreasing order of crowding distance */
void crowding_fill (population *mixed_pop, population *new_pop, int count, int front_size, list *elite)
{
    int *dist;
    list *temp;
    int i, j;
    assign_crowding_distance_list (mixed_pop, elite->child, front_size);
    dist = (int *)malloc(front_size*sizeof(int));
    temp = elite->child;
    for (j=0; j<front_size; j++)
    {
        dist[j] = temp->index;
        temp = temp->child;
    }
    quicksort_dist (mixed_pop, dist, front_size);
    for (i=count, j=front_size-1; i<popsize; i++, j--)
    {
        copy_ind(&mixed_pop->ind[dist[j]], &new_pop->ind[i]);
    }
    free (dist);
    return;
}
