
#include <algorithm>
#include <iostream>
#include "../rand.h"
#include "LinearWeightsValueFunction.h"

using namespace std;

LinearWeightsValueFunction::LinearWeightsValueFunction(int v)
{
  init();
  setValueFunction(v);
}

LinearWeightsValueFunction::~LinearWeightsValueFunction()
{
  if(m_w)
    delete m_w;
}

void LinearWeightsValueFunction::init()
{
  m_w = NULL;
}

void LinearWeightsValueFunction::setValueFunction(int v)
{
  switch(v){

  case LINEAR:
    m_value_function = &linear_value_function;
    break;

  case QUADRATIC:
    m_value_function = &quadratic_value_function;
    break;

  case L4:
    m_value_function = &l4_value_function;
    break;

  case TCHEBYCHEFF:
    m_value_function = &tchebycheff_value_function;
    break;

  case QUADRATIC_TCHEBYCHEFF:
    m_value_function = &quadratic_tchebycheff_value_function;
    break;
    
  default:
    m_value_function = &linear_value_function;
    break;
  }
}

double LinearWeightsValueFunction::getValue(double* obj, int objsize) const
{
  return (*m_value_function)(obj, objsize, m_w);
}


void LinearWeightsValueFunction::setWeights(const string& weightstring)
{  
  int size = count(weightstring.begin(),weightstring.end(),',')+1;

  if(!size)
    return;
  
  m_w = new double[size];
  
  unsigned int start = 0, end = 0, i = 0;
  
  while((end = weightstring.find(',',start)) != string::npos){
    m_w[i++] = atof(weightstring.substr(start,end-start).c_str());
    start = end + 1;
  }
  m_w[i] = atof(weightstring.substr(start).c_str());
}

bool LinearWeightsValueFunction::generateWeights(int nobj)
{
  m_w = new double[nobj];
  double sum = 0;

  for(int i = 0; i < nobj; i++){
    m_w[i] = rndreal(0,1);
    sum += m_w[i];
  } 
  for(int i = 0; i < nobj; i++)
    m_w[i] /= sum;

  return true;
}

bool LinearWeightsValueFunction::setReady(int nobj)
{
  if(m_w)
    return true;
  return generateWeights(nobj);
}
