#ifndef PREFERENCERANKER_H
#define PREFERENCERANKER_H

#define MIN_DISTANCE 1e-4

#include "Ranker.h"
extern "C" {
#include "svm_rank/svm_light/svm_common.h"
}

class PreferenceRanker : public Ranker{

  LEARN_PARM learn_parm;
  KERNEL_PARM kernel_parm;

  MODEL * m_model;
  DOC** m_examples;
  long m_num_examples;
  double* m_targets;

  DOC*** m_train_examples;
  double** m_train_targets;
  long* m_num_train_examples;

  DOC*** m_test_examples;
  double** m_test_targets;
  long* m_num_test_examples;

  long m_max_feature_id;

  // k for cross validation model selection (0 = loo)
  int m_cv_k;

  // whether to perform model selection 
  bool m_do_model_selection;

 public:
  PreferenceRanker(int start, int argc, char **argv);
  ~PreferenceRanker();
  void setPreferences(population *pop, int start, int popsize, int objsize, bool rankerprefs); 
  double preference(double* obj, int objsize, bool rankerprefs); 
  static void print_help();

 protected:
  void free_examples(DOC** examples, long num_examples);
  void init();
  void setRankingPreferences(population *pop, int start, int popsize, int objsize); 
  double train(population *pop, int start, int popsize, int objsize);
  double do_model_selection();
  void do_model_evaluation(double* best_performance,
			   int* best_kernel_type,
			   int* best_degree,
			   double* best_gamma); 
  double do_cross_validation(); 
  void updateSvmProblem(DOC*** examples_p,
			double** targets_p,
			long * num_examples_p,
			long qid,
			population *pop, 
			int popstart, 
			int popsize, 
			int objsize);
  DOC* create_instance(int instnum, double* obj, int objsize, int qid);
  void updateCVProblems(population *pop, int popstart, int popsize, int objsize);
  void train(DOC** examples, double* targets, long num_examples);
  double test(DOC** examples, double* targets, long num_examples);
  void parse_command_line(int start, int argc,char *argv[], long *verbosity,
			  LEARN_PARM *learn_parm, KERNEL_PARM *kernel_parm);
};

#include <iostream>

void copy_learn_parm(LEARN_PARM *src_learn_parm, LEARN_PARM *dst_learn_parm);
void write_examples(std::ostream& out, DOC** examples, double* targets, long num_examples);

#endif




