#ifndef RANKER_H
#define RANKER_H

#include <map>
#include "UserPreference.h"
#include "../global.h"

#ifndef MAXFLOAT
#define MAXFLOAT	3.40282347e+38F
#endif

typedef std::multimap<double,int> PreferenceMap;

class Ranker{

 protected:

  UserPreference m_user_preference;

  // wether to use trained predictor or user preference
  bool m_use_predictor;
  
  // always use gold preference function (to estimate upper bound)
  bool m_use_gold_preference; 

  // threshold for deciding when to stop training
  double m_results_threshold;

  // max number of examples for iteration
  int m_max_iteration_examples;
  
  // max number of iterations
  int m_max_iterations;

  // number of queries 
  int m_queries_number;

  // number of generations between two training iterations
  int m_generations4iteration;

  int m_curr_iteration;


  // number of generations to skip before starting training 
  int m_skip_num_generations;

  // minimum allowed diversity for keeping training example 
  double m_min_allowed_diversity;

  // estimate of nadir points for objective normalization
  double* m_nadir_points;

 public:
  Ranker(int* start, int argc, char **argv);
  virtual ~Ranker();
  virtual bool update(population *pop, int start, int popsize, int objsize);
  virtual double train(population *pop, int start, int popsize, int objsize) = 0;
  virtual double preference(double* obj, int objsize, bool rankerprefs) = 0;
  double userPreference(double* obj, int objsize);
  double userPreference(double* obj1, double* obj2, int objsize);
  void setUserPreferences(population *pop, int objsize, int popsize, int* trainsize);
  virtual void setPreferences(population *pop, list* front, int objsize, bool rankerprefs);
  virtual void setPreferences(population *pop, int start, int popsize, int objsize, bool rankerprefs);
  virtual void preference_ranking(population *mixed_pop, 
				  list* front, int rank, 
				  population *new_pop, int popsize, 
				  int objsize, int* i);
  bool useRanker(int curr_generation) const;
  bool trainRanker(int curr_generation) const;
  bool setReady(int nobj) {return m_user_preference.setReady(nobj);}
  static void print_help();
  static void exit_with_help();
  int getQueriesNumber() const {return m_queries_number;}
  void decrementQueriesNumber() {m_queries_number--;}
  void normalizeObjectives(population *pop, int popsize, int objsize);
  static bool twinPopulation(population *pop, int objsize, int popsize);
  static int firstRedundant(population *pop, int objsize, int popsize, double min_allowed_diversity);

protected:
  virtual void init();
  virtual void parse_command_line(int* start, int argc, char **argv);
  void estimateNadirPoints(population *pop, int popsize, int objsize);
};


double distance(double* x, double* y, int size);

#endif
