#ifndef REGRESSIONRANKER_H
#define REGRESSIONRANKER_H

#include "Ranker.h"
#include "libsvm-2.89/svm.h"

class RegressionRanker : public Ranker{

  struct svm_problem m_prob;
  struct svm_model *m_model;
  struct svm_node *m_x_space;
  struct svm_parameter m_param;
  
  // k for cross validation model selection (0 = loo)
  int m_cv_k;

  // whether to perform model selection 
  bool m_do_model_selection;

  // whether to perform regression on priority vector
  bool m_use_priority_vector;

 public:
  RegressionRanker(int start, int argc, char **argv, bool use_priority_vector=false);
  ~RegressionRanker();
  double train(population *pop, int start, int popsize, int objsize);
  double preference(double* obj, int objsize, bool rankerprefs);
  void setPreferences(population *pop, int start, int popsize, int objsize, bool rankerprefs);
  static void print_help();

 protected:
  void init();
  static svm_node* createSvmNode(double* obj, int objsize);
  static void deleteSvmNode(svm_node* x);
  void setPriorityVectorPreferences(population *pop, int start, int popsize, int objsize);
  void preference2priority(population *pop, int start, int popsize) const;
  void updateSvmProblem(population *pop, int start, int popsize, int objsize);
  void parse_command_line(int start, int argc, char **argv);
  double do_model_selection(const svm_problem& prob, svm_parameter& param, double* target);
  double do_cross_validation(const svm_problem& prob, const svm_parameter& param, int k, double* target) const;
  void do_model_evaluation(const svm_problem& prob, 
			   const svm_parameter& param, 
			   int k, double* target,
			   double* best_performance,
			   int* best_kernel_type,
			   int* best_degree,
			   double* best_gamma) const;
  double average_output(const svm_problem& prob) const;
};

#endif
