#include <map>
#include <iostream>
#include <fstream>
#include "StructPreferenceRanker.h"
extern "C" {
#include "svm_rank/svm_light/svm_common.h"
}
using namespace std;




StructPreferenceRanker::StructPreferenceRanker(int start, int argc, char **argv) :
  Ranker(&start, argc, argv)
{  
  svm_struct_learn_api_init(argc,argv);

  parse_command_line(start, argc,argv,&verbosity,
		     &struct_verbosity,&struct_parm,&learn_parm,
		     &kernel_parm,&alg_type);

  init();

  // orig params needed to reset to original values at each training iteration
  copy_params(&struct_parm, &struct_parm_orig, &learn_parm, &learn_parm_orig);
}

StructPreferenceRanker::~StructPreferenceRanker()
{
  free_struct_sample(sample);
  free_struct_model(structmodel);
  
  svm_struct_learn_api_exit();

  for(int k = 0; k < m_cv_k; k++){
    free_struct_sample(trainsamples[k]);
    free_struct_sample(testsamples[k]);
  }
  delete trainsamples;
  delete testsamples;
}


void StructPreferenceRanker::init()
{
  Ranker::init();
  sample.n=0;
  sample.examples=NULL;
  numinstances=0;

  trainsamples = new SAMPLE[m_cv_k];
  testsamples = new SAMPLE[m_cv_k];
  
  for(int k = 0; k < m_cv_k; k++){
    trainsamples[k].n = 0;
    trainsamples[k].examples=NULL;
    testsamples[k].n = 0;
    testsamples[k].examples=NULL;
  } 

  structmodel.w = NULL;
  structmodel.svm_model = NULL;
  structmodel.sizePsi = 0;
}

double StructPreferenceRanker::preference(double* obj, int objsize, bool rankerprefs) 
{
  double pref;
  PATTERN x;
  
  if(!m_use_gold_preference && rankerprefs){
    create_pattern(&x, obj, objsize, 0, 0);    
    LABEL y=classify_struct_example(x,&structmodel,&struct_parm);    
    pref = y.target[0];
    free_label(y);
    delete_pattern(x);
  }
  else{    
    pref = userPreference(obj, objsize);
  }
  return pref;
}

void StructPreferenceRanker::create_pattern(PATTERN* x, double* obj, int objsize, long exnum, long qid)
{  
  DOC* doc = create_instance(obj, objsize, exnum, qid);

  // create pattern
  x->doc=(DOC **)my_malloc(sizeof(DOC *));  
  x->doc[0]=doc;
  x->totdoc=1;  
}

void StructPreferenceRanker::delete_pattern(PATTERN x)
{
  free_pattern(x);
}


void StructPreferenceRanker::setPreferences(population *pop, int start, int popsize, int objsize, bool rankerprefs) 
{
  if(!rankerprefs)
    setRankingPreferences(pop, start, popsize, objsize);
  else
    Ranker::setPreferences(pop, start, popsize, objsize, rankerprefs);
}

void StructPreferenceRanker::setRankingPreferences(population *pop, int start, int popsize, int objsize) 
{
  // init ranking preferences to 0
  for(int i = start; i < popsize; i++)
    // init preference for current individual
    pop->ind[i].preference = 0;    
  

  for(int i = start; i < popsize; i++)
    for(int j = i+1; j < popsize; j++)
      {
	double pref =  userPreference(pop->ind[i].obj,pop->ind[j].obj, objsize);

	if(pref <= 0)
	  pop->ind[i].preference--;
	if(pref >= 0)
	  pop->ind[j].preference--;
      }	
}


void StructPreferenceRanker::wait_any_key()
{
  printf("\n(more)\n");
  (void)getc(stdin);
}


void StructPreferenceRanker::parse_command_line(int start, int argc,char *argv[],
					  long *verbosity,long *struct_verbosity, 
					  STRUCT_LEARN_PARM *struct_parm,
					  LEARN_PARM *learn_parm, KERNEL_PARM *kernel_parm,
					  int *alg_type)
{
  long i;
  char type[100];
  
  /* set default */
  (*alg_type)=DEFAULT_ALG_TYPE;
  struct_parm->C=-0.01;
  struct_parm->slack_norm=1;
  struct_parm->epsilon=DEFAULT_EPS;
  struct_parm->custom_argc=0;
  struct_parm->loss_function=DEFAULT_LOSS_FCT;
  struct_parm->loss_type=DEFAULT_RESCALING;
  struct_parm->newconstretrain=100;
  struct_parm->ccache_size=5;
  struct_parm->batch_size=100;

  strcpy (learn_parm->predfile, "trans_predictions");
  strcpy (learn_parm->alphafile, "");
  (*verbosity)=0;/*verbosity for svm_light*/
  (*struct_verbosity)=1; /*verbosity for struct learning portion*/
  learn_parm->biased_hyperplane=1;
  learn_parm->remove_inconsistent=0;
  learn_parm->skip_final_opt_check=0;
  learn_parm->svm_maxqpsize=10;
  learn_parm->svm_newvarsinqp=0;
  learn_parm->svm_iter_to_shrink=-9999;
  learn_parm->maxiter=100000;
  learn_parm->kernel_cache_size=40;
  learn_parm->svm_c=99999999;  /* overridden by struct_parm->C */
  learn_parm->eps=0.001;       /* overridden by struct_parm->epsilon */
  learn_parm->transduction_posratio=-1.0;
  learn_parm->svm_costratio=1.0;
  learn_parm->svm_costratio_unlab=1.0;
  learn_parm->svm_unlabbound=1E-5;
  learn_parm->epsilon_crit=0.001;
  learn_parm->epsilon_a=1E-10;  /* changed from 1e-15 */
  learn_parm->compute_loo=0;
  learn_parm->rho=1.0;
  learn_parm->xa_depth=0;
  kernel_parm->kernel_type=0;
  kernel_parm->poly_degree=3;
  kernel_parm->rbf_gamma=1.0;
  kernel_parm->coef_lin=1;
  kernel_parm->coef_const=1;
  strcpy(kernel_parm->custom,"empty");
  strcpy(type,"c");

  m_do_model_selection = false;
  m_cv_k = 0;

  for(i=start;(i<argc) && ((argv[i])[0] == '-');i++) {
    switch ((argv[i])[1]) 
      { 
      case '?': print_help(); exit(0);
      case 'a': i++; strcpy(learn_parm->alphafile,argv[i]); break;
      case 'c': i++; struct_parm->C=atof(argv[i]); break;
      case 'p': i++; struct_parm->slack_norm=atol(argv[i]); break;
      case 'e': i++; struct_parm->epsilon=atof(argv[i]); break;
      case 'k': i++; struct_parm->newconstretrain=atol(argv[i]); break;
      case 'h': i++; learn_parm->svm_iter_to_shrink=atol(argv[i]); break;
      case '#': i++; learn_parm->maxiter=atol(argv[i]); break;
      case 'm': i++; learn_parm->kernel_cache_size=atol(argv[i]); break;
      case 'w': i++; (*alg_type)=atol(argv[i]); break;
      case 'o': i++; struct_parm->loss_type=atol(argv[i]); break;
      case 'n': i++; learn_parm->svm_newvarsinqp=atol(argv[i]); break;
      case 'q': i++; learn_parm->svm_maxqpsize=atol(argv[i]); break;
      case 'l': i++; struct_parm->loss_function=atol(argv[i]); break;
      case 'f': i++; struct_parm->ccache_size=atol(argv[i]); break;
      case 'b': i++; struct_parm->batch_size=atof(argv[i]); break;
      case 't': i++; kernel_parm->kernel_type=atol(argv[i]); break;
      case 'd': i++; kernel_parm->poly_degree=atol(argv[i]); break;
      case 'g': i++; kernel_parm->rbf_gamma=atof(argv[i]); break;
      case 's': i++; kernel_parm->coef_lin=atof(argv[i]); break;
      case 'r': i++; kernel_parm->coef_const=atof(argv[i]); break;
      case 'u': i++; strcpy(kernel_parm->custom,argv[i]); break;
      case '-': strcpy(struct_parm->custom_argv[struct_parm->custom_argc++],argv[i]);i++; strcpy(struct_parm->custom_argv[struct_parm->custom_argc++],argv[i]);break; 
      case 'v': i++; (*struct_verbosity)=atol(argv[i]); break;
      case 'y': i++; (*verbosity)=atol(argv[i]); break;
      case 'M': m_do_model_selection=true; break;
      case 'V': i++; m_cv_k = atoi(argv[i]); break;
      default: printf("\nUnrecognized option %s!\n\n",argv[i]);
	       print_help();
	       exit(0);
      }
  }

  if(m_do_model_selection && !m_cv_k) {
    printf("\nCannot perform model selection with 0 folds.\n"); 
    wait_any_key();
    print_help();
    exit(0);
  }

  if(learn_parm->svm_iter_to_shrink == -9999) {
    learn_parm->svm_iter_to_shrink=100;
  }

  if((learn_parm->skip_final_opt_check) 
     && (kernel_parm->kernel_type == LINEAR)) {
    printf("\nIt does not make sense to skip the final optimality check for linear kernels.\n\n");
    learn_parm->skip_final_opt_check=0;
  }    
  if((learn_parm->skip_final_opt_check) 
     && (learn_parm->remove_inconsistent)) {
    printf("\nIt is necessary to do the final optimality check when removing inconsistent \nexamples.\n");
    wait_any_key();
    print_help();
    exit(0);
  }    
  if((learn_parm->svm_maxqpsize<2)) {
    printf("\nMaximum size of QP-subproblems not in valid range: %ld [2..]\n",learn_parm->svm_maxqpsize); 
    wait_any_key();
    print_help();
    exit(0);
  }
  if((learn_parm->svm_maxqpsize<learn_parm->svm_newvarsinqp)) {
    printf("\nMaximum size of QP-subproblems [%ld] must be larger than the number of\n",learn_parm->svm_maxqpsize); 
    printf("new variables [%ld] entering the working set in each iteration.\n",learn_parm->svm_newvarsinqp); 
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->svm_iter_to_shrink<1) {
    printf("\nMaximum number of iterations for shrinking not in valid range: %ld [1,..]\n",learn_parm->svm_iter_to_shrink);
    wait_any_key();
    print_help();
    exit(0);
  }
  if(struct_parm->C<0) {
    printf("\nYou have to specify a value for the parameter '-c' (C>0)!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(((*alg_type) < 0) || (((*alg_type) > 5) && ((*alg_type) != 9))) {
    printf("\nAlgorithm type must be either '0', '1', '2', '3', '4', or '9'!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->transduction_posratio>1) {
    printf("\nThe fraction of unlabeled examples to classify as positives must\n");
    printf("be less than 1.0 !!!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->svm_costratio<=0) {
    printf("\nThe COSTRATIO parameter must be greater than zero!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(struct_parm->epsilon<=0) {
    printf("\nThe epsilon parameter must be greater than zero!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if((struct_parm->ccache_size<=0) && ((*alg_type) == 4)) {
    printf("\nThe cache size must be at least 1!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(((struct_parm->batch_size<=0) || (struct_parm->batch_size>100))  
     && ((*alg_type) == 4)) {
    printf("\nThe batch size must be in the interval ]0,100]!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if((struct_parm->slack_norm<1) || (struct_parm->slack_norm>2)) {
    printf("\nThe norm of the slacks must be either 1 (L1-norm) or 2 (L2-norm)!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if((struct_parm->loss_type != SLACK_RESCALING) 
     && (struct_parm->loss_type != MARGIN_RESCALING)) {
    printf("\nThe loss type must be either 1 (slack rescaling) or 2 (margin rescaling)!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->rho<0) {
    printf("\nThe parameter rho for xi/alpha-estimates and leave-one-out pruning must\n");
    printf("be greater than zero (typically 1.0 or 2.0, see T. Joachims, Estimating the\n");
    printf("Generalization Performance of an SVM Efficiently, ICML, 2000.)!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if((learn_parm->xa_depth<0) || (learn_parm->xa_depth>100)) {
    printf("\nThe parameter depth for ext. xi/alpha-estimates must be in [0..100] (zero\n");
    printf("for switching to the conventional xa/estimates described in T. Joachims,\n");
    printf("Estimating the Generalization Performance of an SVM Efficiently, ICML, 2000.)\n");
    wait_any_key();
    print_help();
    exit(0);
  }

  parse_struct_parameters(struct_parm);
}


void StructPreferenceRanker::print_help()
{
  Ranker::print_help();

  printf("\nSVM-struct learning module: %s, %s, %s\n",INST_NAME,INST_VERSION,INST_VERSION_DATE);
  printf("   includes SVM-struct %s for learning complex outputs, %s\n",STRUCT_VERSION,STRUCT_VERSION_DATE);
  printf("   includes SVM-light %s quadratic optimizer, %s\n",VERSION,VERSION_DATE);
  copyright_notice();
  printf("   usage: svm_struct_learn [options] example_file model_file\n\n");
  printf("Arguments:\n");
  printf("         example_file-> file with training data\n");
  printf("         model_file  -> file to store learned decision rule in\n");

  printf("General Options:\n");
  printf("         -?          -> this help\n");
  printf("         -v [0..3]   -> verbosity level (default 1)\n");
  printf("         -y [0..3]   -> verbosity level for svm_light (default 0)\n");
  printf("Learning Options:\n");
  printf("         -c float    -> C: trade-off between training error\n");
  printf("                        and margin (default 0.01)\n");
  printf("         -p [1,2]    -> L-norm to use for slack variables. Use 1 for L1-norm,\n");
  printf("                        use 2 for squared slacks. (default 1)\n");
  printf("         -o [1,2]    -> Rescaling method to use for loss.\n");
  printf("                        1: slack rescaling\n");
  printf("                        2: margin rescaling\n");
  printf("                        (default %d)\n",DEFAULT_RESCALING);
  printf("         -l [0..]    -> Loss function to use.\n");
  printf("                        0: zero/one loss\n");
  printf("                        ?: see below in application specific options\n");
  printf("                        (default %d)\n",DEFAULT_LOSS_FCT);
  printf("Optimization Options (see [2][5]):\n");
  printf("         -w [0,..,9] -> choice of structural learning algorithm (default %d):\n",(int)DEFAULT_ALG_TYPE);
  printf("                        0: n-slack algorithm described in [2]\n");
  printf("                        1: n-slack algorithm with shrinking heuristic\n");
  printf("                        2: 1-slack algorithm (primal) described in [5]\n");
  printf("                        3: 1-slack algorithm (dual) described in [5]\n");
  printf("                        4: 1-slack algorithm (dual) with constraint cache [5]\n");
  printf("                        9: custom algorithm in svm_struct_learn_custom.c\n");
  printf("         -e float    -> epsilon: allow that tolerance for termination\n");
  printf("                        criterion (default %f)\n",DEFAULT_EPS);
  printf("         -k [1..]    -> number of new constraints to accumulate before\n"); 
  printf("                        recomputing the QP solution (default 100)\n");
  printf("                        (-w 0 and 1 only)\n");
  printf("         -f [5..]    -> number of constraints to cache for each example\n");
  printf("                        (default 5) (used with -w 4)\n");
  printf("         -b [1..100] -> percentage of training set for which to refresh cache\n");
  printf("                        when no epsilon violated constraint can be constructed\n");
  printf("                        from current cache (default 100%%) (used with -w 4)\n");
  printf("SVM-light Options for Solving QP Subproblems (see [3]):\n");
  printf("         -n [2..q]   -> number of new variables entering the working set\n");
  printf("                        in each svm-light iteration (default n = q). \n");
  printf("                        Set n < q to prevent zig-zagging.\n");
  printf("         -m [5..]    -> size of svm-light cache for kernel evaluations in MB\n");
  printf("                        (default 40) (used only for -w 1 with kernels)\n");
  printf("         -h [5..]    -> number of svm-light iterations a variable needs to be\n"); 
  printf("                        optimal before considered for shrinking (default 100)\n");
  printf("         -# int      -> terminate svm-light QP subproblem optimization, if no\n");
  printf("                        progress after this number of iterations.\n");
  printf("                        (default 100000)\n");
  printf("Kernel Options:\n");
  printf("         -t int      -> type of kernel function:\n");
  printf("                        0: linear (default)\n");
  printf("                        1: polynomial (s a*b+c)^d\n");
  printf("                        2: radial basis function exp(-gamma ||a-b||^2)\n");
  printf("                        3: sigmoid tanh(s a*b + c)\n");
  printf("                        4: user defined kernel from kernel.h\n");
  printf("         -d int      -> parameter d in polynomial kernel\n");
  printf("         -g float    -> parameter gamma in rbf kernel\n");
  printf("         -s float    -> parameter s in sigmoid/poly kernel\n");
  printf("         -r float    -> parameter c in sigmoid/poly kernel\n");
  printf("         -u string   -> parameter of user defined kernel\n");
  printf("Output Options:\n");
  printf("         -a string   -> write all alphas to this file after learning\n");
  printf("                        (in the same order as in the training set)\n");
  printf("Application-Specific Options:\n");
  printf("         -V int      -> n-fold cross validation (default=0 -> no cv)\n");
  printf("         -M          -> do model selection (default=false)\n");

  print_struct_help();
  wait_any_key();

  printf("\nMore details in:\n");
  printf("[1] T. Joachims, Learning to Align Sequences: A Maximum Margin Aproach.\n");
  printf("    Technical Report, September, 2003.\n");
  printf("[2] I. Tsochantaridis, T. Joachims, T. Hofmann, and Y. Altun, Large Margin\n");
  printf("    Methods for Structured and Interdependent Output Variables, Journal\n");
  printf("    of Machine Learning Research (JMLR), Vol. 6(Sep):1453-1484, 2005.\n");
  printf("[3] T. Joachims, Making Large-Scale SVM Learning Practical. Advances in\n");
  printf("    Kernel Methods - Support Vector Learning, B. Schölkopf and C. Burges and\n");
  printf("    A. Smola (ed.), MIT Press, 1999.\n");
  printf("[4] T. Joachims, Learning to Classify Text Using Support Vector\n");
  printf("    Machines: Methods, Theory, and Algorithms. Dissertation, Kluwer,\n");
  printf("    2002.\n");
  printf("[5] T. Joachims, T. Finley, Chun-Nam Yu, Cutting-Plane Training of Structural\n");
  printf("    SVMs, Machine Learning Journal, to appear.\n");
}



void print_sample(ostream& out, SAMPLE& s)
{
  out << s.n << endl;
  for(int i = 0; i < s.n; i++){
    out << "EXAMPLE "  << i << endl; 
    print_example(out, s.examples[i]);
  }  
}

void print_example(ostream& out, EXAMPLE& ex)
{
  out << "LABEL" << endl; 
  print_label(out, ex.y);
  out << "PATTERN" << endl;
  print_pattern(out, ex.x);
}

void print_label(ostream& out, LABEL& y)
{
  out << "loss=" << y.loss << endl;
  out << "totdoc=" << y.totdoc << endl;
  out << "factors=";
  for(int i = 0; i < y.totdoc; i++)
    out << " " << y.factor[i];
  out << "\ntarget=";
  for(int i = 0; i < y.totdoc; i++)
    out << " " << y.target[i];
  out << endl;
}

void print_pattern(ostream& out, PATTERN& x)
{
  out << "scaling=" << x.scaling << endl;
  out << "totdoc=" << x.totdoc << endl;
  for(int i = 0; i < x.totdoc; i++)
    print_doc(out, x.doc[i]);
}
void print_doc(ostream& out, DOC* doc)
{
  out << "docnum=" << doc->docnum << endl;
  out << "queryid=" << doc->queryid << endl;
  out << "costfactor=" << doc->costfactor << endl;
  out << "slackid=" << doc->slackid << endl;
  out << "kernelid=" << doc->kernelid << endl;
  print_svector(out, doc->fvec);
}

void print_svector(ostream& out, SVECTOR* v)
{
  for(;v;v=v->next) {
    out << "factor=" << v->factor << endl;
    if(v->kernel_id) 
      out << "qid=" << v->kernel_id << endl;
    for (int j=0; (v->words[j]).wnum; j++) 
      out << (v->words[j]).wnum << ":" << (v->words[j]).weight << " ";
    out << endl;
    if(v->userdefined)
      out << "userdefined=" << v->userdefined << endl;
    out << "twonorm_sq=" << v->twonorm_sq << endl;
    out << "size=" << v->size << endl;
    if(v->dense)
      for(int j=0; j <= v->size; j++)
	out << v->dense[j] << " ";
    out << endl;
  }
}

void print_struct_parm(ostream& out, STRUCT_LEARN_PARM *struct_parm)
{
  out << struct_parm->C << "\t"
      << struct_parm->slack_norm << "\t"
      << struct_parm->epsilon << "\t"
      << struct_parm->custom_argc << "\t"
      << struct_parm->loss_function << "\t" 
      << struct_parm->loss_type << "\t"
      << struct_parm->newconstretrain << "\t"
      << struct_parm->ccache_size << "\t"
      << struct_parm->batch_size << "\n";
}


void print_learn_parm(ostream& out, LEARN_PARM *learn_parm)
{
  out
    << learn_parm->type << " "
    << learn_parm->svm_c << " "
    << learn_parm->eps << " "
    << learn_parm->svm_costratio << " "
    << learn_parm->transduction_posratio << " "
    << learn_parm->biased_hyperplane << " "
    << learn_parm->sharedslack << " "
    << learn_parm->svm_maxqpsize << " "
    << learn_parm->svm_newvarsinqp << " "
    << learn_parm->kernel_cache_size << " "
    << learn_parm->epsilon_crit << " "
    << learn_parm->epsilon_shrink << " "
    << learn_parm->svm_iter_to_shrink << " "
    << learn_parm->maxiter << " "
    << learn_parm->remove_inconsistent << " "
    << learn_parm->skip_final_opt_check << " "
    << learn_parm->compute_loo << " "
    << learn_parm->rho << " "
    << learn_parm->xa_depth << " "
    << learn_parm->predfile << " "
    << learn_parm->alphafile << " "
    << learn_parm->epsilon_const << " "
    << learn_parm->epsilon_a << " "
    << learn_parm->opt_precision << " "
    << learn_parm->svm_c_steps << " "
    << learn_parm->svm_c_factor << " "
    << learn_parm->svm_costratio_unlab << " "
    << learn_parm->svm_unlabbound << " "
    //<< learn_parm->*svm_cost << " "
    << learn_parm->totwords << endl;
}

void print_kernel_parm(ostream& out, KERNEL_PARM *kernel_parm)
{
  out
    << kernel_parm->kernel_type << " "
    << kernel_parm->poly_degree << " "
    << kernel_parm->rbf_gamma << " "
    << kernel_parm->coef_lin << " "
    << kernel_parm->coef_const << " "
    << kernel_parm->custom << " "
    //<< kernel_parm->*gram_matrix << " "
    << kernel_parm->totwords << endl;
}

void copy_params(STRUCT_LEARN_PARM *struct_parm_src, STRUCT_LEARN_PARM *struct_parm_dst, 
		 LEARN_PARM *learn_parm_src, LEARN_PARM *learn_parm_dst)
{
  struct_parm_dst->C = struct_parm_src->C;
  struct_parm_dst->slack_norm = struct_parm_src->slack_norm;
  struct_parm_dst->epsilon = struct_parm_src->epsilon;
  struct_parm_dst->custom_argc = struct_parm_src->custom_argc;
  struct_parm_dst->loss_function = struct_parm_src->loss_function;
  struct_parm_dst->loss_type = struct_parm_src->loss_type;
  struct_parm_dst->newconstretrain = struct_parm_src->newconstretrain;
  struct_parm_dst->ccache_size = struct_parm_src->ccache_size;
  struct_parm_dst->batch_size = struct_parm_src->batch_size;

  learn_parm_dst->type = learn_parm_src->type;
  learn_parm_dst->svm_c = learn_parm_src->svm_c;
  learn_parm_dst->eps = learn_parm_src->eps;
  learn_parm_dst->svm_costratio = learn_parm_src->svm_costratio;
  learn_parm_dst->transduction_posratio = learn_parm_src->transduction_posratio;
  learn_parm_dst->biased_hyperplane = learn_parm_src->biased_hyperplane;
  learn_parm_dst->sharedslack = learn_parm_src->sharedslack;
  learn_parm_dst->svm_maxqpsize = learn_parm_src->svm_maxqpsize;
  learn_parm_dst->svm_newvarsinqp = learn_parm_src->svm_newvarsinqp;
  learn_parm_dst->kernel_cache_size = learn_parm_src->kernel_cache_size;
  learn_parm_dst->epsilon_crit = learn_parm_src->epsilon_crit;
  learn_parm_dst->epsilon_shrink = learn_parm_src->epsilon_shrink;
  learn_parm_dst->svm_iter_to_shrink = learn_parm_src->svm_iter_to_shrink;
  learn_parm_dst->maxiter = learn_parm_src->maxiter;
  learn_parm_dst->remove_inconsistent = learn_parm_src->remove_inconsistent;
  learn_parm_dst->skip_final_opt_check = learn_parm_src->skip_final_opt_check;
  learn_parm_dst->compute_loo = learn_parm_src->compute_loo;
  learn_parm_dst->rho = learn_parm_src->rho;
  learn_parm_dst->xa_depth = learn_parm_src->xa_depth;
  //learn_parm_dst->predfile = learn_parm_src->predfile;
  //learn_parm_dst->alphafile = learn_parm_src->alphafile;
  learn_parm_dst->epsilon_const = learn_parm_src->epsilon_const;
  learn_parm_dst->epsilon_a = learn_parm_src->epsilon_a;
  learn_parm_dst->opt_precision = learn_parm_src->opt_precision;
  learn_parm_dst->svm_c_steps = learn_parm_src->svm_c_steps;
  learn_parm_dst->svm_c_factor = learn_parm_src->svm_c_factor;
  learn_parm_dst->svm_costratio_unlab = learn_parm_src->svm_costratio_unlab;
  learn_parm_dst->svm_unlabbound = learn_parm_src->svm_unlabbound;
  //learn_parm_dst->*svm_cost = learn_parm_src->*svm_cost; (created and destroyed withing learning function)
  learn_parm_dst->totwords = learn_parm_src->totwords;
}


void StructPreferenceRanker::create_ex(EXAMPLE* ex, population *pop, int currstart, 
				 int popsize, int objsize, int* instnum, int qid)
{
  double   *labels;
  DOC      **instances;
  long     n = popsize; /* number of instances */

 // add ranking example to sample
  PATTERN* x=&(ex->x);
  LABEL* y=&(ex->y);

  create_instances(&instances, &labels, pop, currstart, popsize, objsize, instnum, qid);

  x->totdoc=n;
  x->doc=(DOC**)my_malloc(sizeof(DOC *)*x->totdoc);
  y->totdoc=n;
  y->target=(double *)my_malloc(sizeof(double)*y->totdoc);
  y->factor=(double *)my_malloc(sizeof(double)*y->totdoc);
  y->loss=0;

  for(int i = 0; i < n; i++){
    x->doc[i] = instances[i];
    y->target[i] = labels[i];
  }

  free(instances);
  free(labels);
}

void StructPreferenceRanker::create_instances(DOC ***docs, double **label, 
					population *pop, int currstart, 
					int popsize, int objsize, int* instnum, int qid)
{
  int n = popsize-currstart;

  (*docs) = (DOC **)my_malloc(sizeof(DOC *)*n);    /* feature vectors */
  (*label) = (double *)my_malloc(sizeof(double)*n); /* target values */



  for(int i = currstart; i < popsize; i++){
    (*label)[i] = pop->ind[i].preference;
    (*docs)[i] = create_instance(pop->ind[i].obj, objsize, *instnum, qid);
    (*instnum)++;

  }

  //DEBUG
//   FILE* outfile = fopen("debug.train","a");
//   for(int i = currstart; i < popsize; i++){
//     fprintf(outfile,"%.8g qid:%d ", pop->ind[i].preference, qid); 
//     for(int j = 0; j < objsize; j++)
//       fprintf(outfile," %ld:%.8g", j+1, pop->ind[i].obj[j]); 
//     fprintf(outfile,"\n");
//   }
//   fclose(outfile);  

    
}  



// DEBUG
// void StructPreferenceRanker::create_instances(DOC ***docs, double **label, 
// 					population *pop, int currstart, 
// 					int popsize, int objsize, int* instnum, int qid)
// {
//   int n = popsize-currstart;

//   (*docs) = (DOC **)my_malloc(sizeof(DOC *)*n);    /* feature vectors */
//   (*label) = (double *)my_malloc(sizeof(double)*n); /* target values */

//   for(int i = currstart; i < popsize; i++){
//     (*label)[i] = pop->ind[i].preference;
//     (*docs)[i] = create_instance(pop->ind[i].obj, objsize, *instnum, qid);
//     (*instnum)++;
//   }

//   // compute averages and stddevs
//   double* mean = new double[objsize];
//   double* stddev = new double[objsize];
//   double* zscore = new double[objsize];

//   for(int j = 0; j < objsize; j++)
//     mean[j] = 0;

//   for(int i = currstart; i < popsize; i++)
//     for(int j = 0; j < objsize; j++)
//       mean[j] += pop->ind[i].obj[j]; 

//   for(int j = 0; j < objsize; j++)
//     mean[j] /= (popsize-currstart);

//   for(int i = currstart; i < popsize; i++)
//     for(int j = 0; j < objsize; j++)
//       stddev[j] += (pop->ind[i].obj[j]-mean[j])*(pop->ind[i].obj[j]-mean[j]);
  
//   for(int j = 0; j < objsize; j++)
//     stddev[j] = sqrt(stddev[j]/(popsize-currstart));
    
//   FILE* outfile = fopen("debug.train","a");

//   for(int i = currstart; i < popsize; i++){
    
//     for(int j = 0; j < objsize; j++)
//       zscore[j] = (pop->ind[i].obj[j]-mean[j])/stddev[j]; 

//     fprintf(outfile,"%.8g qid:%d ", userPreference(zscore, objsize), qid); 

//     for(int j = 0; j < objsize; j++)
//       fprintf(outfile," %d:%.8g", j+1, zscore[j]);

//     fprintf(outfile,"\n");
//   }

//   delete[] mean;
//   delete[] stddev;
//   delete[] zscore;

//   fclose(outfile);
    
// }  

DOC* StructPreferenceRanker::create_instance(double* obj, int objsize, int instnum, int qid)
{
  WORD* words = (WORD *)my_malloc(sizeof(WORD)*(objsize+1));

  DOC* doc;

  for(int i = 0; i < objsize; i++){
    words[i].wnum = i+1;
    words[i].weight = obj[i];
  }
  words[objsize].wnum = 0;

  // create doc
  doc = create_example(instnum, qid, 0,1.,create_svector(words,(char*)"",1.0));  
  free(words);

  return doc;
}

void StructPreferenceRanker::adjustSample(SAMPLE* psample)
{
  PATTERN  *x;
  LABEL    *y;
  long     totpairs, sumtotpairs, i, j, k;

  SAMPLE currsample=*psample;
 
 /* Remove all features with numbers larger than num_features, if
     num_features is set to a positive value. This is important for
     svm_struct_classify. */
  if(struct_parm.num_features > 0) 
    for(k=0;k<currsample.n;k++) 
      for(i=0;i<currsample.examples[k].x.totdoc;i++)
	for(j=0;currsample.examples[k].x.doc[i]->fvec->words[j].wnum;j++) 
	  if(currsample.examples[k].x.doc[i]->fvec->words[j].wnum>struct_parm.num_features) {
	    currsample.examples[k].x.doc[i]->fvec->words[j].wnum=0;
	    currsample.examples[k].x.doc[i]->fvec->words[j].weight=0;
	  }

  /* add label factors for easy computation of feature vectors */
  sumtotpairs=0;
  for(k=0;k<currsample.n;k++) {
    x=&currsample.examples[k].x;
    y=&currsample.examples[k].y;
    // moved to: create_ex
    //y->factor=(double *)my_malloc(sizeof(double)*y->totdoc);
    for(i=0;i<y->totdoc;i++) 
      y->factor[i]=0;
    totpairs=0;
    for(i=0;i<y->totdoc;i++) {
      for(j=0;j<y->totdoc;j++) {
	if(y->target[i] > y->target[j]) {
	  totpairs++;
	  y->factor[i]+=0.5;
	  y->factor[j]-=0.5;
	}
      }
    }
    sumtotpairs+=totpairs;
    x->scaling=1.0/(double)totpairs; /* for FRACSWAPPEDPAIRS */
  }

  /* modify scaling factor */
  if(struct_parm.loss_function == SWAPPEDPAIRS) {
    for(k=0;k<currsample.n;k++) {
      x=&currsample.examples[k].x;
      x->scaling=1.0;
    }
  }
  else if(struct_parm.loss_function == FRACSWAPPEDPAIRS) {
    /* already set to correct value above */
  }
  else {
    printf("Unknown loss function type: %d\n",struct_parm.loss_function);
    exit(1);
  }
  for(k=0;k<currsample.n;k++) {
    x=&currsample.examples[k].x;
    y=&currsample.examples[k].y;
    for(i=0;i<y->totdoc;i++) 
      y->factor[i]*=x->scaling;
  }
}


void StructPreferenceRanker::updateSvmProblem(population *pop, int currstart, int popsize, int objsize)
{  
  // update set size
  sample.n++;

  // allocate memory
  if(sample.examples)
    sample.examples = (EXAMPLE *)realloc(sample.examples,sizeof(EXAMPLE)*sample.n);
  else
    sample.examples = (EXAMPLE *)my_malloc(sizeof(EXAMPLE)*sample.n);


  // create ranking example from individuals 
  create_ex(&sample.examples[sample.n-1], pop, currstart, popsize, objsize, &numinstances, sample.n);

  adjustSample(&sample);
}


void StructPreferenceRanker::updateSvmProblem(SAMPLE* psample, population *pop, 
					int currstart, int popsize, int objsize,
					int numfolds, int fold, bool testfold)
{  
  // update set size
  psample->n++;

  // allocate memory
  if(psample->examples)
    psample->examples = (EXAMPLE *)realloc(psample->examples,sizeof(EXAMPLE)*psample->n);
  else
    psample->examples = (EXAMPLE *)my_malloc(sizeof(EXAMPLE)*psample->n);


  // create ranking example from individuals 
  create_ex(&(psample->examples[psample->n-1]), pop, currstart, popsize, objsize, &numinstances, 
	    psample->n, numfolds, fold, testfold);

  adjustSample(psample);
}


void StructPreferenceRanker::create_ex(EXAMPLE* ex, population *pop, int currstart, 
				 int popsize, int objsize, int* instnum, int qid,
				 int numfolds, int fold, bool testfold)
{
  long n;
  double   *labels;
  DOC      **instances;

  // add ranking example to sample
  PATTERN* x=&(ex->x);
  LABEL* y=&(ex->y);

  n = create_instances(&instances, &labels, pop, currstart, popsize, objsize, 
		       instnum, qid, numfolds, fold, testfold);

  x->totdoc=n;
  x->doc=(DOC**)my_malloc(sizeof(DOC *)*x->totdoc);
  y->totdoc=n;
  y->target=(double *)my_malloc(sizeof(double)*y->totdoc);
  y->factor=(double *)my_malloc(sizeof(double)*y->totdoc);
  y->loss=0;

  for(int i = 0; i < n; i++){
    x->doc[i] = instances[i];
    y->target[i] = labels[i];
  }

  free(instances);
  free(labels);
}

long StructPreferenceRanker::create_instances(DOC ***docs, double **label, 
					population *pop, int currstart, 
					int popsize, int objsize, int* instnum, int qid,
					int numfolds, int fold, bool testfold)
{
  int n = 0;
  // compute maximal size
  int maxn = (popsize-currstart) / m_cv_k;
  maxn++;
  if(!testfold)
    maxn *= (m_cv_k-1);

  (*docs) = (DOC **)my_malloc(sizeof(DOC *)*maxn);    /* feature vectors */
  (*label) = (double *)my_malloc(sizeof(double)*maxn); /* target values */

  //DEBUG
  //FILE* outfile = fopen("debug.train","a");

  for(int i = currstart; i < popsize; i++){
    if((i - currstart) % numfolds != fold){
      if(testfold)
	continue;
    }
    else
      if(!testfold)
	continue;      
    (*label)[n] = pop->ind[i].preference;
    (*docs)[n] = create_instance(pop->ind[i].obj, objsize, *instnum, qid);
    (*instnum)++;
    n++;

    // DEBUG
//     fprintf(outfile,"%.8g qid:%d ", pop->ind[i].preference, qid); 
//     for(int j = 0; j < objsize; j++)
//       fprintf(outfile," %ld:%.8g", j+1, pop->ind[i].obj[j]); 
//     fprintf(outfile,"\n");
  }

  //DEBUG
  //fclose(outfile);  

  return n;
}  


void StructPreferenceRanker::updateCVProblems(population *pop, int currstart, int popsize, int objsize)
{

  // update train folds
  for(int k = 0; k < m_cv_k; k++)
    updateSvmProblem(&(trainsamples[k]), pop, currstart, popsize, objsize, m_cv_k, k, false);

  // update test folds
  for(int k = 0; k < m_cv_k; k++)
    updateSvmProblem(&(testsamples[k]), pop, currstart, popsize, objsize, m_cv_k, k, true);
}



double StructPreferenceRanker::train(population *pop, int start, int popsize, int objsize)
{
  double results = 0.;

  // add current individuals to training set
  updateSvmProblem(pop, start, popsize, objsize);

  if(m_cv_k)
    // add current individuals to folds
    updateCVProblems(pop, start, popsize, objsize);
	
  cout << "\n\nStarting ranker training\n" << endl;

  if(m_do_model_selection){
    //double c = struct_parm.C;
    //struct_parm.C = 1;
    results = do_model_selection();

    std::cout << "choosing: kernel=" << kernel_parm.kernel_type
	      << " degree=" << kernel_parm.poly_degree
	      << " gamma=" << kernel_parm.rbf_gamma << std::endl;
    //m_do_model_selection = false;
    //struct_parm.C = c;
  }
  else if(m_cv_k)
    // evaluate current parameter setting
    results = do_cross_validation();

  // train model
  train(&sample);

  cout << "\n\nFinished ranker training\n" << endl;
  
  // DEBUG
  //write_model("debug.model",structmodel.svm_model);

  return results;
}


void StructPreferenceRanker::train(SAMPLE* psample)
{
  // reset params to original value
  copy_params(&struct_parm_orig, &struct_parm, &learn_parm_orig, &learn_parm);
	
  // clear current model
  free_struct_model(structmodel);


  /* modify scaling factor */
  if(struct_parm.loss_function == SWAPPEDPAIRS) {
    long sumtotpairs=0;
    LABEL    *y;
    for(int k=0;k<(*psample).n;k++){ 
      y=&(*psample).examples[k].y;
      for(int i=0;i<y->totdoc;i++) 
	for(int j=0;j<y->totdoc;j++) 
	  if(y->target[i] > y->target[j]) 	  
	    sumtotpairs++;
    }
    struct_parm.epsilon*=(double)sumtotpairs/(double)(*psample).n; /* adjusting eps */
  }

  /* Do the learning and return structmodel. */
  if(alg_type == 0)
    svm_learn_struct(*psample,&struct_parm,&learn_parm,&kernel_parm,&structmodel,NSLACK_ALG);
  else if(alg_type == 1)
    svm_learn_struct(*psample,&struct_parm,&learn_parm,&kernel_parm,&structmodel,NSLACK_SHRINK_ALG);
  else if(alg_type == 2)
    svm_learn_struct_joint(*psample,&struct_parm,&learn_parm,&kernel_parm,&structmodel,ONESLACK_PRIMAL_ALG);
  else if(alg_type == 3)
    svm_learn_struct_joint(*psample,&struct_parm,&learn_parm,&kernel_parm,&structmodel,ONESLACK_DUAL_ALG);
  else if(alg_type == 4)
    svm_learn_struct_joint(*psample,&struct_parm,&learn_parm,&kernel_parm,&structmodel,ONESLACK_DUAL_CACHE_ALG);
  else if(alg_type == 9)
    svm_learn_struct_joint_custom(*psample,&struct_parm,&learn_parm,&kernel_parm,&structmodel);
  else
    exit(1);
  
  if(structmodel.svm_model->kernel_parm.kernel_type == LINEAR) { /* linear kernel */
    /* compute weight vector */
    add_weight_vector_to_linear_model(structmodel.svm_model);
    structmodel.w=structmodel.svm_model->lin_weights;

    cout << "normalized learned polynomial:" << endl; 
    double sum = 0.;
    for(int i = 1; i < structmodel.svm_model->totwords; i++)
      sum += structmodel.svm_model->lin_weights[i];
    
    for(int i = 1; i < structmodel.svm_model->totwords; i++){
      if(i>1)
	cout << " + ";
      cout << structmodel.svm_model->lin_weights[i] / sum;
      cout << " * x" << i-1;
    }
    cout << endl;
  }
}

double StructPreferenceRanker::test(SAMPLE* psample)
{
  SAMPLE testsample = *psample;
  LABEL y;
  double avgloss=0,l;
  
  for(int i=0;i<testsample.n;i++) {
  
    y=classify_struct_example(testsample.examples[i].x,&structmodel,&struct_parm);

    l=loss(testsample.examples[i].y,y,&struct_parm);

    avgloss+=l;

    //eval_prediction(i,testsample.examples[i],y,&model,&sparm,&teststats);
    
    free_label(y);
  }  
  avgloss/=testsample.n;

  return avgloss;
}

double StructPreferenceRanker::do_model_selection()
{
  double best_performance = -FLT_MAX;
  int best_kernel_type = kernel_parm.kernel_type;
  double best_gamma = kernel_parm.rbf_gamma;
  int best_degree = kernel_parm.poly_degree;
  
  //int maxdegree = 4;
  int maxdegree = 2;
  int numgammas = 7;
  double gammas[] = {1e-3,1e-2,1e-1,1,1e1,1e2,1e3};
  
  // linear
  kernel_parm.kernel_type = 0;
  do_model_evaluation(&best_performance, &best_kernel_type, &best_degree, &best_gamma);
  if(best_performance >= m_results_threshold)
    return best_performance;

  // polynomial
  kernel_parm.kernel_type = 1;
  for(int d = 2; d <= maxdegree; d++){
    kernel_parm.poly_degree = d;
    do_model_evaluation(&best_performance, &best_kernel_type, &best_degree, &best_gamma);
    if(best_performance >= m_results_threshold)
      return best_performance;
  }

  // gaussian
  kernel_parm.kernel_type = 2;
  for(int g = 0; g < numgammas; g++){
    kernel_parm.rbf_gamma = gammas[g];
    do_model_evaluation(&best_performance, &best_kernel_type, &best_degree, &best_gamma);
    if(best_performance >= m_results_threshold)
      return best_performance;
  }
         
  kernel_parm.kernel_type = best_kernel_type;
  kernel_parm.poly_degree = best_degree;
  kernel_parm.rbf_gamma = best_gamma;  

  return best_performance;
}

void StructPreferenceRanker::do_model_evaluation(double* best_performance,
					   int* best_kernel_type,
					   int* best_degree,
					   double* best_gamma) 
{
  
  double curr_performance = do_cross_validation();	
     
  std::cout << "kernel=" << kernel_parm.kernel_type
       << " degree=" << kernel_parm.poly_degree
       << " gamma=" << kernel_parm.rbf_gamma 
       << " cv_performance=" << curr_performance << std::endl;
  
  if(curr_performance > *best_performance){
    *best_performance = curr_performance;
    *best_kernel_type = kernel_parm.kernel_type;
    *best_degree = kernel_parm.poly_degree;
    *best_gamma = kernel_parm.rbf_gamma;
  }
}


double StructPreferenceRanker::do_cross_validation() 
{
  double avgloss = 0.;

  for(int k = 0; k < m_cv_k; k++){
    train(&(trainsamples[k]));
    avgloss += test(&(testsamples[k]));
  }

  return 1 - (avgloss/m_cv_k);
}


