#ifndef STRUCTPREFERENCERANKER_H
#define STRUCTPREFERENCERANKER_H

#include "Ranker.h"
extern "C" {
#include "svm_rank/svm_struct_api.h"
#include "svm_rank/svm_struct/svm_struct_learn.h"
}

class StructPreferenceRanker : public Ranker{

  SAMPLE sample;  /* training sample */
  LEARN_PARM learn_parm;
  KERNEL_PARM kernel_parm;
  STRUCT_LEARN_PARM struct_parm;
  STRUCTMODEL structmodel;
  int alg_type;
  int numinstances;

  LEARN_PARM learn_parm_orig;
  STRUCT_LEARN_PARM struct_parm_orig;

  // k for cross validation model selection (0 = loo)
  int m_cv_k;

  // whether to perform model selection 
  bool m_do_model_selection;

  /* for cv */
  SAMPLE* trainsamples;
  SAMPLE* testsamples;


 public:
  StructPreferenceRanker(int start, int argc, char **argv);
  ~StructPreferenceRanker();
  double preference(double* obj, int objsize, bool rankerprefs);
  void setPreferences(population *pop, int start, int popsize, int objsize, bool rankerprefs);
  static void print_help();

 protected:
  void init();
  static void create_pattern(PATTERN* x, double* obj, int objsize, long exnum, long queryid);
  static void delete_pattern(PATTERN x);
  void updateSvmProblem(population *pop, int start, int popsize, int objsize);
  void parse_command_line(int start, int argc,char *argv[],
			  long *verbosity,long *struct_verbosity, 
			  STRUCT_LEARN_PARM *struct_parm,
			  LEARN_PARM *learn_parm, KERNEL_PARM *kernel_parm,
			  int *alg_type);
  void setRankingPreferences(population *pop, int start, int popsize, int objsize);
  void create_instances(DOC ***docs, double **label, 
			population *pop, int currstart, 
			int popsize, int objsize, int* instnum, int qid);
  static DOC* create_instance(double* obj, int objsize, int instnum, int qid);
  void create_ex(EXAMPLE* ex, population *pop, int currstart, 
		 int popsize, int objsize, int* instnum, int qid);
  void adjustSample(SAMPLE* psample);
  void updateSvmProblem(SAMPLE* psample, population *pop, 
			int currstart, int popsize, int objsize,
			int numfolds, int fold, bool testfold);
  void create_ex(EXAMPLE* ex, population *pop, int currstart, 
		 int popsize, int objsize, int* instnum, int qid,
		 int numfolds, int fold, bool testfold);
  long create_instances(DOC ***docs, double **label, 
			population *pop, int currstart, 
			int popsize, int objsize, int* instnum, int qid,
			int numfolds, int fold, bool testfold);
  void updateCVProblems(population *pop, int currstart, int popsize, int objsize);
  double train(population *pop, int start, int popsize, int objsize);
  void train(SAMPLE* psample);
  double test(SAMPLE* psample);
  double do_model_selection();
  void do_model_evaluation(double* best_performance,
			   int* best_kernel_type,
			   int* best_degree,
			   double* best_gamma);
  double do_cross_validation();
  static void wait_any_key();
  
};

void print_sample(std::ostream& out, SAMPLE& s);
void print_example(std::ostream& out, EXAMPLE& ex);
void print_label(std::ostream& out, LABEL& y);
void print_pattern(std::ostream& out, PATTERN& x);
void print_doc(std::ostream& out, DOC* doc);
void print_svector(std::ostream& out, SVECTOR* v);
void print_struct_parm(std::ostream& out, STRUCT_LEARN_PARM *struct_parm);
void print_learn_parm(std::ostream& out, LEARN_PARM *learn_parm);
void print_kernel_parm(std::ostream& out, KERNEL_PARM *kernel_parm);
void copy_params(STRUCT_LEARN_PARM *struct_parm_src, STRUCT_LEARN_PARM *struct_parm_dst, 
		 LEARN_PARM *learn_parm_src, LEARN_PARM *learn_parm_dst);
#endif
