#include "UserPreference.h"
#include "PolynomialValueFunction.h"
#include "LinearWeightsValueFunction.h"
#include <iostream>
#include <stdlib.h>
#include <math.h>

#define tolerance 0.001 

using namespace std;

UserPreference::UserPreference()
{
  init();
}

UserPreference::~UserPreference()
{
  if(m_value_function)
    delete m_value_function;
}

void UserPreference::init()
{
  m_value_function = NULL;
}

void UserPreference::setValueFunction(int v)
{
  if(v <= ValueFunction::QUADRATIC_TCHEBYCHEFF)
    m_value_function = new LinearWeightsValueFunction(v);
  else if( v == ValueFunction::POLYNOMIAL)
    m_value_function = new PolynomialValueFunction();
  else{
    cerr << "ERROR: unknown value function " << v << " ... exiting" << endl;
    exit(0);
  }    
}

double UserPreference::getValue(double* obj, int objsize) const
{
  return m_value_function->getValue(obj, objsize);
}

double UserPreference::getPreference(double* obj1, double* obj2, int objsize) const
{
  return m_value_function->getValue(obj1, objsize) - m_value_function->getValue(obj2, objsize);
}

// double UserPreference::getPreference(double* obj1, double* obj2, int objsize) const
// {
//   double pref = m_value_function->getValue(obj1, objsize) - m_value_function->getValue(obj2, objsize);
  
//   if( fabs(pref)/min(m_value_function->getValue(obj1, objsize), m_value_function->getValue(obj2, objsize)) 
//       < tolerance)
//     return 0;
//   return pref;
// }

void UserPreference::setWeights(const string& weightstring)
{
  LinearWeightsValueFunction* f = dynamic_cast<LinearWeightsValueFunction*>(m_value_function); 

  if(!f){
    cerr << "ERROR: weights can be set from string only in LinearWeights functions" << endl;
    exit(0);
  }

  f->setWeights(weightstring);
}

void UserPreference::setPolynomialDegree(int d)
{
  PolynomialValueFunction* f = dynamic_cast<PolynomialValueFunction*>(m_value_function); 
  
  if(!f){
    cerr << "ERROR: setting degree only for Polynomial function" << endl;
    exit(0);
  }

  f->setDegree(d);

}

void UserPreference::setPolynomialWeightsNum(int n)
{
  PolynomialValueFunction* f = dynamic_cast<PolynomialValueFunction*>(m_value_function); 
  
  if(!f){
    cerr << "ERROR: setting weights num only for Polynomial function" << endl;
    exit(0);
  }

  f->setWeightsNum(n);

}

void UserPreference::setPolynomial(const std::string& pol)
{
  PolynomialValueFunction* f = dynamic_cast<PolynomialValueFunction*>(m_value_function); 
  
  if(!f){
    cerr << "ERROR: setting polynomial only for Polynomial function" << endl;
    exit(0);
  }

  f->readPolynomial(pol);
}
