#include <math.h>
#include <limits.h>
#include "value_functions.h"

#define KV 2

#ifndef MAXFLOAT
#define MAXFLOAT	3.40282347e+38F
#endif

/* linear user preference */
double linear_value_function(double* obj, int nobj, double* w)
{
  int i;
  double val = 0.;

  for (i=0; i<nobj; i++)
    val += w[i]*obj[i];

  return val;
}


/* quadratic user preference */
double quadratic_value_function(double* obj, int nobj, double* w)
{
  int i;
  double val = 0;

  for (i=0; i<nobj; i++)
    val += w[i]*(1-obj[i])*(1-obj[i]);

  return KV - sqrt(val);
}


/* L4 metric user preference */
double l4_value_function(double* obj, int nobj, double* w)
{
  int i;
  double val = 0,tmp;

  for (i=0; i<nobj; i++){
    tmp = (1-obj[i]);
    val += w[i]*pow(tmp,4);
  }

  return KV - pow(val,1./4);
}

/* Tchebycheff metric user preference */
double tchebycheff_value_function(double* obj, int nobj, double* w)
{
  int i;
  double val = -MAXFLOAT, tmp;

  for (i=0; i<nobj; i++){
    tmp =  w[i]*(1-obj[i]);
    if(tmp > val)
      val = tmp;
  }
  
  return KV - val;
}

/* combined quadratic Tchebycheff metric user preference */
double quadratic_tchebycheff_value_function(double* obj, int nobj, double* w)
{ 
  return (quadratic_value_function(obj,nobj,w) + tchebycheff_value_function(obj,nobj,w))/2.; 
}

  
#ifdef STANDALONE_VALUE_FUNCTION
#include <stdio.h>
#include <stdlib.h>
int main(int argc, char** argv)
{
  int func;
  int nobj;
  int i;
  double* obj;
  double* w;
  double value=0;
  
  printf("Value function:\n");
  printf("       0 -- LINEAR\n");
  printf("       1 -- QUADRATIC\n");
  printf("       2 -- L4\n");
  printf("       3 -- TCHEBYCHEFF\n");
  printf("       4 -- QUADRATIC_TCHEBYCHEFF\n");
  
  scanf("%d",&func);

  printf("Number of objectives:");

  scanf("%d",&nobj);

  obj = (double*)malloc(sizeof(double)*nobj);
  w = (double*)malloc(sizeof(double)*nobj);
  
  for(i = 0; i < nobj; i++){
    printf("Obj%d: ", i);
    scanf("%lf",&obj[i]);
    printf("w%d: ", i);
    scanf("%lf",&w[i]);
  }
  
  if(func==0)
    value = linear_value_function(obj,nobj,w);
  if(func==1)
    value = quadratic_value_function(obj,nobj,w);
  if(func==2)
    value = l4_value_function(obj,nobj,w);
  if(func==3)
    value = tchebycheff_value_function(obj,nobj,w);
  if(func==4)
    value = quadratic_tchebycheff_value_function(obj,nobj,w);
  
  printf("value=%lf",value);
    
  free(obj);
  free(w);
}

#endif
