/* Test problem definitions */

# include <stdio.h>
# include <stdlib.h>
# include <math.h>
# include <values.h>

# include "global.h"
# include "rand.h"
# include "problemdef.h"

/*  Test problem SCH1
    # of real variables = 1
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void sch1
  (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    obj[0] = pow(xreal[0],2.0);
    obj[1] = pow((xreal[0]-2.0),2.0);
    return;
}


/*  Test problem SCH2
    # of real variables = 1
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void sch2
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    if (xreal[0]<=1.0)
    {
        obj[0] = -xreal[0];
        obj[1] = pow((xreal[0]-5.0),2.0);
        return;
    }
    if (xreal[0]<=3.0)
    {
        obj[0] = xreal[0]-2.0;
        obj[1] = pow((xreal[0]-5.0),2.0);
        return;
    }
    if (xreal[0]<=4.0)
    {
        obj[0] = 4.0-xreal[0];
        obj[1] = pow((xreal[0]-5.0),2.0);
        return;
    }
    obj[0] = xreal[0]-4.0;
    obj[1] = pow((xreal[0]-5.0),2.0);
    return;
}


/*  Test problem FON
    # of real variables = n
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void fon
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double s1, s2;
    int i;
    s1 = s2 = 0.0;
    for (i=0; i<nreal; i++)
    {
        s1 += pow((xreal[i]-(1.0/sqrt((double)nreal))),2.0);
        s2 += pow((xreal[i]+(1.0/sqrt((double)nreal))),2.0);
    }
    obj[0] = 1.0 - exp(-s1);
    obj[1] = 1.0 - exp(-s2);
    return;
}


/*  Test problem KUR
    # of real variables = 3
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void kur
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    int i;
    double res1, res2;
    res1 = -0.2*sqrt((xreal[0]*xreal[0]) + (xreal[1]*xreal[1]));
    res2 = -0.2*sqrt((xreal[1]*xreal[1]) + (xreal[2]*xreal[2]));
    obj[0] = -10.0*( exp(res1) + exp(res2));
    obj[1] = 0.0;
    for (i=0; i<3; i++)
    {
        obj[1] += pow(fabs(xreal[i]),0.8) + 5.0*sin(pow(xreal[i],3.0));
    }
    return;
}


/*  Test problem POL
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void pol
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double a1, a2, b1, b2;
    a1 = 0.5*sin(1.0) - 2.0*cos(1.0) + sin(2.0) - 1.5*cos(2.0);
    a2 = 1.5*sin(1.0) - cos(1.0) + 2.0*sin(2.0) - 0.5*cos(2.0);
    b1 = 0.5*sin(xreal[0]) - 2.0*cos(xreal[0]) + sin(xreal[1]) - 1.5*cos(xreal[1]);
    b2 = 1.5*sin(xreal[0]) - cos(xreal[0]) + 2.0*sin(xreal[1]) - 0.5*cos(xreal[1]);
    obj[0] = 1.0 + pow((a1-b1),2.0) + pow((a2-b2),2.0);
    obj[1] = pow((xreal[0]+3.0),2.0) + pow((xreal[1]+1.0),2.0);
    return;
}


/*  Test problem VNT
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 3
    # of constraints = 0
    */

void vnt
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    obj[0] = 0.5*(xreal[0]*xreal[0] + xreal[1]*xreal[1]) + sin(xreal[0]*xreal[0] + xreal[1]*xreal[1]);
    obj[1] = (pow((3.0*xreal[0] - 2.0*xreal[1] + 4.0),2.0))/8.0 + (pow((xreal[0]-xreal[1]+1.0),2.0))/27.0 + 15.0;
    obj[2] = 1.0/(xreal[0]*xreal[0] + xreal[1]*xreal[1] + 1.0) - 1.1*exp(-(xreal[0]*xreal[0] + xreal[1]*xreal[1]));
    return;
}


/*  Test problem ZDT1
    # of real variables = 30
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void zdt1
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double f1, f2, g, h;
    int i;
    f1 = xreal[0];
    g = 0.0;
    for (i=1; i<30; i++)
    {
        g += xreal[i];
    }
    g = 9.0*g/29.0;
    g += 1.0;
    h = 1.0 - sqrt(f1/g);
    f2 = g*h;
    obj[0] = f1;
    obj[1] = f2;
    return;
}


/*  Test problem ZDT2
    # of real variables = 30
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void zdt2
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double f1, f2, g, h;
    int i;
    f1 = xreal[0];
    g = 0.0;
    for (i=1; i<30; i++)
    {
        g += xreal[i];
    }
    g = 9.0*g/29.0;
    g += 1.0;
    h = 1.0 - pow((f1/g),2.0);
    f2 = g*h;
    obj[0] = f1;
    obj[1] = f2;
    return;
}


/*  Test problem ZDT3
    # of real variables = 30
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void zdt3
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double f1, f2, g, h;
    int i;
    f1 = xreal[0];
    g = 0.0;
    for (i=1; i<30; i++)
    {
        g += xreal[i];
    }
    g = 9.0*g/29.0;
    g += 1.0;
    h = 1.0 - sqrt(f1/g) - (f1/g)*sin(10.0*PI*f1);
    f2 = g*h;
    obj[0] = f1;
    obj[1] = f2;
    return;
}


/*  Test problem ZDT4
    # of real variables = 10
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void zdt4
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double f1, f2, g, h;
    int i;
    f1 = xreal[0];
    g = 0.0;
    for (i=1; i<10; i++)
    {
        g += xreal[i]*xreal[i] - 10.0*cos(4.0*PI*xreal[i]);
    }
    g += 91.0;
    h = 1.0 - sqrt(f1/g);
    f2 = g*h;
    obj[0] = f1;
    obj[1] = f2;
    return;
}


/*  Test problem ZDT5
    # of real variables = 0
    # of bin variables = 11
    # of bits for binvar1 = 30
    # of bits for binvar2-11 = 5
    # of objectives = 2
    # of constraints = 0
    */

void zdt5
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    int i, j;
    int u[11];
    int v[11];
    double f1, f2, g, h;
    for (i=0; i<11; i++)
    {
        u[i] = 0;
    }
    for (j=0; j<30; j++)
    {
        if (gene[0][j] == 1)
        {
            u[0]++;
        }
    }
    for (i=1; i<11; i++)
    {
        for (j=0; j<4; j++)
        {
            if (gene[i][j] == 1)
            {
                u[i]++;
            }
        }
    }
    f1 = 1.0 + u[0];
    for (i=1; i<11; i++)
    {
        if (u[i] < 5)
        {
            v[i] = 2 + u[i];
        }
        else
        {
            v[i] = 1;
        }
    }
    g = 0;
    for (i=1; i<11; i++)
    {
        g += v[i];
    }
    h = 1.0/f1;
    f2 = g*h;
    obj[0] = f1;
    obj[1] = f2;
    return;
}


/*  Test problem ZDT6
    # of real variables = 10
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 0
    */

void zdt6
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double f1, f2, g, h;
    int i;
    f1 = 1.0 - (exp(-4.0*xreal[0]))*pow((sin(4.0*PI*xreal[0])),6.0);
    g = 0.0;
    for (i=1; i<10; i++)
    {
        g += xreal[i];
    }
    g = g/9.0;
    g = pow(g,0.25);
    g = 1.0 + 9.0*g;
    h = 1.0 - pow((f1/g),2.0);
    f2 = g*h;
    obj[0] = f1;
    obj[1] = f2;
    return;
}


/*  Test problem BNH
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 2
    */

void bnh
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    obj[0] = 4.0*(xreal[0]*xreal[0] + xreal[1]*xreal[1]);
    obj[1] = pow((xreal[0]-5.0),2.0) + pow((xreal[1]-5.0),2.0);
    constr[0] = 1.0 - (pow((xreal[0]-5.0),2.0) + xreal[1]*xreal[1])/25.0;
    constr[1] = (pow((xreal[0]-8.0),2.0) + pow((xreal[1]+3.0),2.0))/7.7 - 1.0;
    return;
}


/*  Test problem OSY
    # of real variables = 6
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 6
    */

void osy
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    obj[0] = -(25.0*pow((xreal[0]-2.0),2.0) + pow((xreal[1]-2.0),2.0) + pow((xreal[2]-1.0),2.0) + pow((xreal[3]-4.0),2.0) + pow((xreal[4]-1.0),2.0));
    obj[1] = xreal[0]*xreal[0] +  xreal[1]*xreal[1] + xreal[2]*xreal[2] + xreal[3]*xreal[3] + xreal[4]*xreal[4] + xreal[5]*xreal[5];
    constr[0] = (xreal[0]+xreal[1])/2.0 - 1.0;
    constr[1] = 1.0 - (xreal[0]+xreal[1])/6.0;
    constr[2] = 1.0 - xreal[1]/2.0 + xreal[0]/2.0;
    constr[3] = 1.0 - xreal[0]/2.0 + 3.0*xreal[1]/2.0;
    constr[4] = 1.0 - (pow((xreal[2]-3.0),2.0))/4.0 - xreal[3]/4.0;
    constr[5] = (pow((xreal[4]-3.0),2.0))/4.0 + xreal[5]/4.0 - 1.0;
    return;
}


/*  Test problem SRN
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 2
    */

void srn
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    obj[0] = 2.0 + pow((xreal[0]-2.0),2.0) + pow((xreal[1]-1.0),2.0);
    obj[1] = 9.0*xreal[0] - pow((xreal[1]-1.0),2.0);
    constr[0] = 1.0 - (pow(xreal[0],2.0) + pow(xreal[1],2.0))/225.0;
    constr[1] = 3.0*xreal[1]/10.0 - xreal[0]/10.0 - 1.0;
    return;
}


/*  Test problem TNK
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 2
    */

void tnk
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    obj[0] = xreal[0];
    obj[1] = xreal[1];
    if (xreal[1] == 0.0)
    {
        constr[0] = -1.0;
    }
    else
    {
        constr[0] = xreal[0]*xreal[0] + xreal[1]*xreal[1] - 0.1*cos(16.0*atan(xreal[0]/xreal[1])) - 1.0;
    }
    constr[1] = 1.0 - 2.0*pow((xreal[0]-0.5),2.0) + 2.0*pow((xreal[1]-0.5),2.0);
    return;
}


/*  Test problem CTP1
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 2
    */

void ctp1
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double g;
    g = 1.0 + xreal[1];
    obj[0] = xreal[0];
    obj[1] = g*exp(-obj[0]/g);
    constr[0] = obj[1]/(0.858*exp(-0.541*obj[0]))-1.0;
    constr[1] = obj[1]/(0.728*exp(-0.295*obj[0]))-1.0;
    return;
}


/*  Test problem CTP2
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 1
    */

void ctp2
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double g;
    double theta, a, b, c, d, e;
    double exp1, exp2;
    theta = -0.2*PI;
    a = 0.2;
    b = 10.0;
    c = 1.0;
    d = 6.0;
    e = 1.0;
    g = 1.0 + xreal[1];
    obj[0] = xreal[0];
    obj[1] = g*(1.0  - sqrt(obj[0]/g));
    exp1 = (obj[1]-e)*cos(theta) - obj[0]*sin(theta);
    exp2 = (obj[1]-e)*sin(theta) + obj[0]*cos(theta);
    exp2 = b*PI*pow(exp2,c);
    exp2 = fabs(sin(exp2));
    exp2 = a*pow(exp2,d);
    constr[0] = exp1/exp2 - 1.0;
    return;
}


/*  Test problem CTP3
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 1
    */

void ctp3
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double g;
    double theta, a, b, c, d, e;
    double exp1, exp2;
    theta = -0.2*PI;
    a = 0.1;
    b = 10.0;
    c = 1.0;
    d = 0.5;
    e = 1.0;
    g = 1.0 + xreal[1];
    obj[0] = xreal[0];
    obj[1] = g*(1.0  - sqrt(obj[0]/g));
    exp1 = (obj[1]-e)*cos(theta) - obj[0]*sin(theta);
    exp2 = (obj[1]-e)*sin(theta) + obj[0]*cos(theta);
    exp2 = b*PI*pow(exp2,c);
    exp2 = fabs(sin(exp2));
    exp2 = a*pow(exp2,d);
    constr[0] = exp1/exp2 - 1.0;
    return;
}


/*  Test problem CTP4
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 1
    */

void ctp4
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double g;
    double theta, a, b, c, d, e;
    double exp1, exp2;
    theta = -0.2*PI;
    a = 0.75;
    b = 10.0;
    c = 1.0;
    d = 0.5;
    e = 1.0;
    g = 1.0 + xreal[1];
    obj[0] = xreal[0];
    obj[1] = g*(1.0  - sqrt(obj[0]/g));
    exp1 = (obj[1]-e)*cos(theta) - obj[0]*sin(theta);
    exp2 = (obj[1]-e)*sin(theta) + obj[0]*cos(theta);
    exp2 = b*PI*pow(exp2,c);
    exp2 = fabs(sin(exp2));
    exp2 = a*pow(exp2,d);
    constr[0] = exp1/exp2 - 1.0;
    return;
}


/*  Test problem CTP5
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 1
    */

void ctp5
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double g;
    double theta, a, b, c, d, e;
    double exp1, exp2;
    theta = -0.2*PI;
    a = 0.1;
    b = 10.0;
    c = 2.0;
    d = 0.5;
    e = 1.0;
    g = 1.0 + xreal[1];
    obj[0] = xreal[0];
    obj[1] = g*(1.0  - sqrt(obj[0]/g));
    exp1 = (obj[1]-e)*cos(theta) - obj[0]*sin(theta);
    exp2 = (obj[1]-e)*sin(theta) + obj[0]*cos(theta);
    exp2 = b*PI*pow(exp2,c);
    exp2 = fabs(sin(exp2));
    exp2 = a*pow(exp2,d);
    constr[0] = exp1/exp2 - 1.0;
    return;
}


/*  Test problem CTP6
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 1
    */

void ctp6
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double g;
    double theta, a, b, c, d, e;
    double exp1, exp2;
    theta = 0.1*PI;
    a = 40.0;
    b = 0.5;
    c = 1.0;
    d = 2.0;
    e = -2.0;
    g = 1.0 + xreal[1];
    obj[0] = xreal[0];
    obj[1] = g*(1.0  - sqrt(obj[0]/g));
    exp1 = (obj[1]-e)*cos(theta) - obj[0]*sin(theta);
    exp2 = (obj[1]-e)*sin(theta) + obj[0]*cos(theta);
    exp2 = b*PI*pow(exp2,c);
    exp2 = fabs(sin(exp2));
    exp2 = a*pow(exp2,d);
    constr[0] = exp1/exp2 - 1.0;
    return;
}


/*  Test problem CTP7
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 1
    */

void ctp7
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double g;
    double theta, a, b, c, d, e;
    double exp1, exp2;
    theta = -0.05*PI;
    a = 40.0;
    b = 5.0;
    c = 1.0;
    d = 6.0;
    e = 0.0;
    g = 1.0 + xreal[1];
    obj[0] = xreal[0];
    obj[1] = g*(1.0  - sqrt(obj[0]/g));
    exp1 = (obj[1]-e)*cos(theta) - obj[0]*sin(theta);
    exp2 = (obj[1]-e)*sin(theta) + obj[0]*cos(theta);
    exp2 = b*PI*pow(exp2,c);
    exp2 = fabs(sin(exp2));
    exp2 = a*pow(exp2,d);
    constr[0] = exp1/exp2 - 1.0;
    return;
}


/*  Test problem CTP8
    # of real variables = 2
    # of bin variables = 0
    # of objectives = 2
    # of constraints = 2
    */

void ctp8
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
    double g;
    double theta, a, b, c, d, e;
    double exp1, exp2;
    g = 1.0 + xreal[1];
    obj[0] = xreal[0];
    obj[1] = g*(1.0  - sqrt(obj[0]/g));
    theta = 0.1*PI;
    a = 40.0;
    b = 0.5;
    c = 1.0;
    d = 2.0;
    e = -2.0;
    exp1 = (obj[1]-e)*cos(theta) - obj[0]*sin(theta);
    exp2 = (obj[1]-e)*sin(theta) + obj[0]*cos(theta);
    exp2 = b*PI*pow(exp2,c);
    exp2 = fabs(sin(exp2));
    exp2 = a*pow(exp2,d);
    constr[0] = exp1/exp2 - 1.0;
    theta = -0.05*PI;
    a = 40.0;
    b = 2.0;
    c = 1.0;
    d = 6.0;
    e = 0.0;
    exp1 = (obj[1]-e)*cos(theta) - obj[0]*sin(theta);
    exp2 = (obj[1]-e)*sin(theta) + obj[0]*cos(theta);
    exp2 = b*PI*pow(exp2,c);
    exp2 = fabs(sin(exp2));
    exp2 = a*pow(exp2,d);
    constr[1] = exp1/exp2 - 1.0;
    return;
}



/*  Test problem RCW
    # of real variables = 4
    # of bin variables = 0
    # of objectives = 3
    # of constraints = 2
    */

void RCW
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  double alpha_c[4] = {8.0,8.0,8.0,8.0};
  double alpha_w[4] = {6.0,6.0,6.0,6.0};
  double alpha_v[4] = {2.0,2.0,2.0,2.0};

  double beta_c[4] = {2.0,10.0,3.0,18.0};
  double beta_w[4] = {3.0,2.0,10.0,8.0};
  double beta_v[4] = {2.0,2.0,6.0,8.0};

  double gamma_c[4] = {2.0,2.0,2.0,2.0};
  double gamma_w[4] = {0.5,0.5,0.5,0.5};
  double gamma_v[4] = {0.5,0.5,0.5,0.5};

  double n[4] = {7,8,7,8};
  
  int i;

  double Vs = 0.;

  obj[0] = 1.; /* Rs */
  obj[1] = 0.; /* Cs */
  obj[2] = 0.; /* Ws */

  for(i = 0; i < 4; i++){
    obj[0] *= (1-pow(1-xreal[i],n[i]));
    obj[1] += n[i]*alpha_c[i]*pow(log10(beta_c[i]/(1-xreal[i])),gamma_c[i]);
    obj[2] += n[i]*alpha_w[i]*pow(log10(beta_w[i]/(1-xreal[i])),gamma_w[i]);
    Vs += n[i]*alpha_v[i]*pow(log10(beta_v[i]/(1-xreal[i])),gamma_v[i]);
  }
  
  constr[0] = 65 - Vs; /* 65 - Vs */
  constr[1] = 12000 - obj[2]*Vs; /* 12000 - Ws*Vs */

  /* normalized values */
  obj[0] = (obj[0] - 0.9)/0.1;
  obj[1] = (obj[1] - 550)/-550;
  obj[2] = (obj[2] - 350)/-350;

  obj[0] = - obj[0]; /* max Rs */

  return;
}

/*  Test problem DTLZ1
    # of real variables = n
    # of bin variables = 0
    # of objectives = m
    # of constraints = 2*n
    */

void dtlz1
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int k,i,j;
  double g = 0.0;

  k = nreal - nobj + 1;

  for (i = nreal - k + 1; i <= nreal; i++)
    g += pow(xreal[i-1]-0.5,2) - cos(20 * PI * (xreal[i-1]-0.5));

  g = 100 *(k + g);

  for (i = 1; i <= nobj; i++) {
    double f = 0.5 * (1 + g);
    for (j = nobj - i; j >= 1; j--)
      f *= xreal[j-1];
    
    if (i > 1)
      f *= 1 - xreal[(nobj - i + 1) - 1];
    
    obj[i-1] = f;
  }

  for(i = 0; i < nreal; i++){
    constr[i]=xreal[i]-0.25;
    constr[nreal+i]=0.75-xreal[i];
  }
}

/*  Test problem DTLZ6 
    # of real variables = n
    # of bin variables = 0
    # of objectives = m
    # of constraints = 2*n
    */

void dtlz2
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int k;
  double g;

  k = nreal - nobj + 1;
  g = 0.0;
  for (int i = nreal - k + 1; i <= nreal; i++)
    g += pow(xreal[i-1]-0.5,2);
  
  for (int i = 1; i <= nobj; i++) {
    double f = (1 + g);
    for (int j = nobj - i; j >= 1; j--)
      f *= cos(xreal[j-1] * PI / 2);
    
    if (i > 1)
      f *= sin(xreal[(nobj - i + 1) - 1] * PI / 2);
    
    obj[i-1] = f;
  }	

  for(int i = 0; i < nreal; i++){
    constr[i]=xreal[i]-0.25;
    constr[nreal+i]=0.75-xreal[i];
  }
}

void dtlz3
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int k;
  double g;

  k = nreal - nobj + 1;
  g = 0.0;
  for (int i = nreal - k + 1; i <= nreal; i++)
    g += pow(xreal[i-1]-0.5,2) - cos( 20 * PI * (xreal[i-1] - 0.5) );
  g = 100 * (k + g);
  
  for (int i = 1; i <= nobj; i++) {
    double f = (1 + g);
    for (int j = nobj - i; j >= 1; j--)
      f *= cos(xreal[j-1] * PI / 2);
    
    if (i > 1)
      f *= sin(xreal[(nobj - i + 1) - 1] * PI / 2);
    
    obj[i-1] = f;
  }

  for(int i = 0; i < nreal; i++){
    constr[i]=xreal[i]-0.25;
    constr[nreal+i]=0.75-xreal[i];
  }
}

void dtlz4
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int k;
  double g;
  int alpha = 100;

  k = nreal - nobj + 1;
  g = 0.0;
  for (int i = nreal - k + 1; i <= nreal; i++)
    g += pow(xreal[i-1]-0.5,2) - cos( 20 * PI * (xreal[i-1] - 0.5) );
  g = 100 * (k + g);
  
  for (int i = 1; i <= nobj; i++) {
    double f = (1 + g);
    for (int j = nobj - i; j >= 1; j--)
      f *= cos( pow(xreal[j-1], alpha) * PI / 2);
    
    if (i > 1)
      f *= sin( pow(xreal[(nobj - i + 1) - 1], alpha) * PI / 2);
    
    obj[i-1] = f;
  }

  for(int i = 0; i < nreal; i++){
    constr[i]=xreal[i]-0.25;
    constr[nreal+i]=0.75-xreal[i];
  }
}


void dtlz4bis
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int k;
  double g;
  int alpha = 100;

  k = nreal - nobj + 1;
  g = 0.0;
  for (int i = nreal - k + 1; i <= nreal; i++)
    g += pow(xreal[i-1]-0.5,2);
  
  for (int i = 1; i <= nobj; i++) {
    double f = (1 + g);
    for (int j = nobj - i; j >= 1; j--)
      f *= cos( pow(xreal[j-1], alpha) * PI / 2);
    
    if (i > 1)
      f *= sin( pow(xreal[(nobj - i + 1) - 1], alpha) * PI / 2);
    
    obj[i-1] = f;
  }

  for(int i = 0; i < nreal; i++){
    constr[i]=xreal[i]-0.25;
    constr[nreal+i]=0.75-xreal[i];
  }
}

/* simply dtlz5 in paper with g = pow(x-0.5,2) */
#define PI_2 1.57079632679489661923
void dtlz5
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int k;
  double g, t;

  double* theta = new double[nobj-1];
  
  k = nreal - nobj + 1;
  g = 0.0;
  for (int i = nreal - k + 1; i <= nreal; i++)
    g += pow(xreal[i-1]-0.5,2);
  
  t= PI /(4 * (1 + g));
  
  theta[0]= PI_2 * xreal[0];
  for (int i = 1; i <= nobj - 2 ; i++)
    theta[i]=  t * (1 + 2 * g * xreal[i]);
  
  for (int i = 1; i <= nobj; i++) {
    double f = (1 + g);
    for (int j = nobj - i; j >= 1; j--)
      f *= cos(theta[j-1]);
    
    if (i > 1)
      f *= sin(theta[(nobj - i + 1) - 1]);
    
    obj[i-1] = f;
  }

  for(int i = 0; i < nreal; i++){
    constr[i]=xreal[i]-0.25;
    constr[nreal+i]=0.75-xreal[i];
  }

  delete theta;
}

/* simply dtlz5 in paper with g = pow(x,0.1); */
void dtlz6
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int k;
  double g, t;

  double* theta = new double[nobj-1];
  
  k = nreal - nobj + 1;
  g = 0.0;
  for (int i = nreal - k + 1; i <= nreal; i++)
    g += pow(xreal[i-1],0.1);
  
  t= PI /(4 * (1 + g));
  
  theta[0]= PI * xreal[0] / 2;
  for (int i = 1; i <= nobj - 2 ; i++)
    theta[i]=  t * (1 + 2 * g * xreal[i]);
  
  for (int i = 1; i <= nobj; i++) {
    double f = (1 + g);
    for (int j = nobj - i; j >= 1; j--)
      f *= cos(theta[j-1]);
    
    if (i > 1)
      f *= sin(theta[(nobj - i + 1) - 1]);
    
    obj[i-1] = f;
  }

  for(int i = 0; i < nreal; i++){
    constr[i]=xreal[i]-0.25;
    constr[nreal+i]=0.75-xreal[i];
  }

  delete theta;
}

/* corresponds to dtlz6 in paper */
void dtlz7
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int i;  
  int k;
  double g, h;
  
  k = nreal - nobj + 1;
  g = 0.0;
  h = 0.0;
  for (i = nreal - k + 1; i <= nreal; i++)
    g += xreal[i-1];
  g= 1 + (9 * g ) / k;
   
  for (i = 1; i <= nobj -1 ; i++)
    obj[i-1]=xreal[i-1];
  
  for (i = 1 ; i< nobj; i++)
    h += xreal[i-1] / (1 + g) * (1 + sin(3 * PI * xreal[i-1]));
  h = nobj - h;
  
  obj[nobj -1] = (1 + g) * h;

  for(int i = 0; i < nreal; i++){
    constr[i]=xreal[i]; 
    constr[nreal+i]=1-xreal[i];
  }
}

/* corresponds to dtlz7 in paper */
void dtlz8
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int i;  
  int j;
  int s,e;
  double minsum;
  
  /* f objectives */
  for (j = 1; j <= nobj; j++){
    obj[j-1]=0.;
    s = floor((j-1)*(double)nreal/nobj)+1;
    e = floor(j*(double)nreal/nobj);
    for (i = s; i <= e; i++)
      obj[j-1] += xreal[i-1];
    obj[j-1] /= floor(nreal/nobj);
  }

  /* x range constraints */
  for(int i = 0; i < nreal; i++){
    constr[i]=xreal[i];
    constr[nreal+i]=1-xreal[i];
  }

  /* gj constraints */
  for (j = 0; j < nobj-1; j++)
    constr[2*nreal+j] = obj[nobj-1] + 4 * obj[j] - 1;

  /* gm constraint */
  minsum = FLT_MAX;
  for (j = 0; j < nobj-1; j++)
    for (i = j+1; i < nobj-1; i++)
      if((obj[i] + obj[j]) < minsum)
	minsum = obj[i] + obj[j];
  constr[2*nreal+nobj-1] = 2 * obj[nobj-1] + minsum - 1;
}

/* multiple knapsack problem */

double* profit=NULL;
double* weight=NULL;
double* capacity=NULL;
double sumweight=0.;

void init_knapsack_problem()
{
  int i,j;
  int nitems=nbin/nobj;
  
  double minweight=1.;

  profit=(double *)malloc(nitems*sizeof(double));
  weight=(double *)malloc(nitems*sizeof(double));
  capacity=(double *)malloc(nobj*sizeof(double));

  printf("Initializing profit for items:\n");
  for (j = 0; j < nitems; j++){
    profit[j] = rndreal(0,1);
    printf("%lf ",profit[j]);
  }
  printf("\nInitializing weight for items:\n");
  for (j = 0; j < nitems; j++){
    weight[j] = rndreal(0,1);
    printf("%lf ",weight[j]);
  }
    
  for (j = 0; j < nitems; j++){
    sumweight+=weight[j];
    if(minweight>weight[j])
      minweight=weight[j];
  }

  printf("\nMinweight=%lf\n", minweight);
  printf("\nSumweight=%lf\n", sumweight);

  printf("\nInitializing knapsacks capacity:\n");
  for (i = 0; i < nobj; i++){
    capacity[i] = rndreal(minweight,sumweight);
    //capacity[i] = rndreal(minweight,sumweight);
    //capacity[i] = rndreal(minweight,sumweight/(nobj));
    //capacity[i] = rndreal(minweight,sumweight/10.);
    printf("%lf ",capacity[i]);
  }
  printf("\n");
  
}

void clear_knapsack_problem()
{
  if(profit)
    free(profit);
  if(weight)
    free(weight);
  if(capacity)
    free(capacity);
}

void knapsack
 (double *xreal, double *xbin, int **gene, double *obj, double *constr)
{
  int i,j;  
  int nitems=nbin/nobj;
  
  /* compute (negated) profits */
  for (i = 0; i < nobj; i++){
    obj[i]=0.;
    for (j = 0; j < nitems; j++)
      obj[i]-=profit[j]*gene[i*nitems+j][0];
  }
  
  /* knapsacks capacity constraints */
  for (i = 0; i < nobj; i++){
    constr[i]=capacity[i];
    for (j = 0; j < nitems; j++)
      constr[i]-=weight[j]*gene[i*nitems+j][0];
  }

  /* non-ubiquitous items constraints */
  for (j = 0; j < nitems; j++){
    constr[nobj+j]=1;
    for (i = 0; i < nobj; i++)
      constr[nobj+j]-=gene[i*nitems+j][0];
  }

}

void init_problem(pt2TestProblem test_problem)
{
  if (test_problem==&knapsack)
    init_knapsack_problem();
}

void clear_problem(pt2TestProblem test_problem)
{
  if (test_problem==&knapsack)
    clear_knapsack_problem();
}
